///////////////////////////////////////////////////////////////////////
/////// FICHIER		: GBTherm10.c
/////// DESCRIPTION	: Thermostat v1.0
/////// PCB				: GBTherm
/////// AUTEUR			: G. B
/////// DATE			: 07/2005
//
/////// Modifications par rapport a la version precedente :
///////		- Cration du programme
///////
/////// Bugs connus :
///////
///////////////////////////////////////////////////////////////////////

#include <16F876.h>
#device ADC=10	// <<<<<<<< Pour acqui 10 bits
#fuses NOPROTECT,NODEBUG,WRT,NOCPD,NOLVP,NOBROWNOUT,NOPUT,NOWDT,HS
// PROTECT			protection Flash totale
// PROTECT_50%		protection Flash de 1000  1FFF
// PROTECT_5%		???
// NOPROTECT		Pas de protection Flash
// DEBUG				RB6 et RB7 utilises pour debugger
// NODEBUG			RB6 et RB7 en utilisation normale
// WRT				Le programme peut crire dans la flash
// NOWRT				Le programme ne peut pas crire dans la flash
// CPD				Mmoire EEprom protge
// NOCPD				Mmoire EEprom dprotge
// LVP				RB3 permet la programmation srie de la PIC
// NOLVP				RB3 en utilisation normale
// BROWNOUT			Reset si Vdd < 4V. Valide PWRTE_ON automatiquement
// NOBROWNOUT		Reset tension hors service
// NOPUT				Dmarrage rapide
// PUT				Dmarrage temporis
// WDT				Watchdog en service
// NOWDT				Watchdog hors service
// LP					Oscillateur basse vitesse (32<F<200Khz)
// XT					Oscilateur moyenne vitesse (200Khz<F<4Mhz)
// HS					Oscillateur haute vitesse (4Mhz<F<20Mhz)
// RC					Oscillateur  rseau RC

#use delay(clock=4000000)

//#use rs232(baud=9600, xmit=PIN_C6, rcv=PIN_C7)  // Jumpers: 8 to 11, 7 to 12

/////////////////////////////////////////////////////////////
///////////////////     Constantes      /////////////////////
/////////////////////////////////////////////////////////////
// Adresse des paramtres sauvegards en EEPROM
#define	NVM_V_PROGRAMME_Hebdo_Byte0	0
#define	NVM_V_REGUL_Hysteresis			21
#define	NVM_V_TEMPNUIT_Byte0				22
#define	NVM_V_TEMPJOUR_Byte0				26
#define	NVM_Checksum						30

#define INTERRUPTS_PER_SECOND 10
#define PERIODE_TIMER1	49964

/////////////////////////////////////////////////////////////
///////////////////     Variables       /////////////////////
/////////////////////////////////////////////////////////////

int	V_HORLOGE_Heures;		// 1 bit = 1 heure	
int	V_HORLOGE_Minutes;	// 1 bit = 1 minute	
int	V_HORLOGE_Secondes;	// 1 bit = 1 seconde	
int	V_HORLOGE_Jour;		// 1 bit = 1 Jour, Lundi = 1
short	V_HORLOGE_SecondePaire;	// Permet clignotements  0.5 Hz
			
float	V_TEMPERATURE;
#define C_TEMP_MoyGliss	10	// Nb pts pour moyenne glissante
float	V_TEMPNUIT;
float	V_TEMPJOUR;
float	V_CONSIGNE_T;
int	V_REGUL_Hysteresis;	// 1 bit = 0,1 degr	 = Delta entre consigne et mesure pour chgt rgulation. 0.2 a 0.9
			
int	V_MODE;
#define C_Regulation		0
#define C_Reglage			1
int	V_MODE_Regul;
#define C_OFF				0
#define C_Auto				1
#define C_ManuNuit		2
#define C_ManuJour		3
#define C_NuitForce		4
#define C_JourForce		5
short	V_ModeOFFForce;
int	V_MODE_Reglage;
#define C_Lundi			0
#define C_Mardi			1
#define C_Mercredi		2
#define C_Jeudi			3
#define C_Vendredi		4
#define C_Samedi			5
#define C_Dimanche		6
#define C_TempNuit		7
#define C_TempJour		8
#define C_Jour				9
#define C_Heures			10
#define C_Minutes			11
#define C_Hysteresis		12
int	V_PROGRAMME_Hebdo[21];	// 3 Octets / jour, 1 bit = 1 heure	
int	V_PlageHoraire;			// Plage horaire du programme en cours de rglage
			
short	V_PROGRAMME_Jour;			// 1 = Jour, 0 = Nuit	Dit si le programme demande Jour ou Nuit
int	V_TIMER_Force_Heure_Fin;	// 1 bit = 1 heure	0  23 h
			
int	V_TOUCHE_Appuyee;				//	= Touche en cours d'appui
#define C_Aucune			0
#define C_Touche_Mode	1
#define C_Touche_Jour	2
#define C_Touche_Nuit	3
#define C_Touche_Reglage	4
int	V_TOUCHE_EtaitDejaAppuyee;	// = Touche qui tait dj appuye au cycle d'avant
//			0 = Aucune
//			1 = C_Touche_Mode		Appui sur la touche MODE
//			2 = C_Touche_Jour		Appui sur la touche JOUR
//			3 = C_Touche_Nuit		Appui sur la touche NUIT
//			4 = C_Touche_Reglage	Appui sur la touche REGLAGE
int	V_TOUCHE_Selectionnee;		// = Touche qui a t appuye puis relche
//			0 = Aucune
//			1 = C_Touche_Mode		Appui sur la touche MODE
//			2 = C_Touche_Jour		Appui sur la touche JOUR
//			3 = C_Touche_Nuit		Appui sur la touche NUIT
//			4 = C_Touche_Reglage	Appui sur la touche REGLAGE
int	V_TOUCHE_AppuiDuree;		// 1 bit = 0,1 seconde	Dure de l'appui en cours
short	V_TOUCHE_AppuiLong;		// 1 = Long		Dit si la touche a t appuye longtemps ou non
#define C_TOUCHE_DureeMiniAppuiLong	10	// 1 bit = 0,1 seconde	Dure mini d'un appui long

int	int_count;    // Number of interrupts left before a second has elapsed

char Str[14];			// pour generation des lignes de characteres a afficher

/////////////////////////////////////////////////////////////
///////////////////  Entrees / Sorties  /////////////////////
/////////////////////////////////////////////////////////////
#define	TOUCHE_Mode		(!input(PIN_C0))
#define	TOUCHE_Jour		(!input(PIN_C5))
#define	TOUCHE_Nuit		(!input(PIN_C2))
#define	TOUCHE_Reglage	(!input(PIN_C1))

#define	LED_Jour			PIN_B5
#define	LED_Nuit			PIN_B6

#define	RELAI_Pilote1	PIN_C4
#define	RELAI_Pilote2	PIN_C3

#define	ENTREE_OFFForce	input(PIN_A4)

#define	NOKIA_Clk		PIN_B0
#define	NOKIA_Data		PIN_B1
#define	NOKIA_D_C		PIN_B2
#define	NOKIA_SCE		PIN_B3
#define	NOKIA_Reset		PIN_B4

/////////////////////////////////////////////////////////////
///////////////////     Fonctions   /////////////////////////
/////////////////////////////////////////////////////////////

#include <../Nokia/Driver C/nokia10.h>

///////////////////     NVM

int NVM_Calcul_Checksum(int start, int end)
{
	int checksum;
	int X;
	
	checksum = 0;
	for (X = start ; X <= end ; X++)
	{
		checksum += READ_EEPROM(X);
	}
	return (~checksum);		// Complment  1
}

void NVM_Params_Save()
{
	int NVM_Octet;
	int *ptr;
	
	for (NVM_Octet=0;NVM_Octet<21;NVM_Octet++)
	{
		WRITE_EEPROM((NVM_V_PROGRAMME_Hebdo_Byte0 + NVM_Octet),V_PROGRAMME_Hebdo[NVM_Octet]);
	}
	
	WRITE_EEPROM(NVM_V_REGUL_Hysteresis,V_REGUL_Hysteresis);
	
	ptr = &V_TEMPNUIT;
	for (NVM_Octet=0;NVM_Octet<4;NVM_Octet++)
	{
		WRITE_EEPROM((NVM_V_TEMPNUIT_Byte0 + NVM_Octet),*(ptr+NVM_Octet));
	}
	
	ptr = &V_TEMPJOUR;
	for (NVM_Octet=0;NVM_Octet<4;NVM_Octet++)
	{
		WRITE_EEPROM((NVM_V_TEMPJOUR_Byte0 + NVM_Octet),*(ptr+NVM_Octet));
	}
	
	WRITE_EEPROM(NVM_Checksum, NVM_Calcul_Checksum(0,NVM_Checksum-1));
}

void NVM_Params_Read()
{
	int NVM_Octet, checksum;
	int *ptr;
	
	for (NVM_Octet=0;NVM_Octet<21;NVM_Octet++)
	{
		V_PROGRAMME_Hebdo[NVM_Octet] = READ_EEPROM(NVM_V_PROGRAMME_Hebdo_Byte0 + NVM_Octet);
	}
	
	V_REGUL_Hysteresis = READ_EEPROM(NVM_V_REGUL_Hysteresis);
	
	ptr = &V_TEMPNUIT;
	for (NVM_Octet=0;NVM_Octet<4;NVM_Octet++)
	{
		*(ptr+NVM_Octet) = READ_EEPROM(NVM_V_TEMPNUIT_Byte0 + NVM_Octet);
	}
	
	ptr = &V_TEMPJOUR;
	for (NVM_Octet=0;NVM_Octet<4;NVM_Octet++)
	{
		*(ptr+NVM_Octet) = READ_EEPROM(NVM_V_TEMPJOUR_Byte0 + NVM_Octet);
	}
	
	Checksum = READ_EEPROM(NVM_Checksum);
	if (Checksum != NVM_Calcul_Checksum(0,NVM_Checksum-1)) // Checksum erron => valeurs par dfaut
	{
		for (NVM_Octet=0;NVM_Octet<21;NVM_Octet++) V_PROGRAMME_Hebdo[NVM_Octet] = 0;
		V_TEMPNUIT = 17.0;
		V_TEMPJOUR = 20.5;
		V_REGUL_Hysteresis = 3;
	}
}

///////////////////////////////////////////////////////////////////////

void Incremente_Horloge()				// Incrm Sec, Min, Heures, Jour
{
	ldiv_t lidiv;
	int X, Y;
	
	V_HORLOGE_Secondes += 1;
	V_HORLOGE_SecondePaire = !V_HORLOGE_SecondePaire; //pour clignotement LEDs
	if (V_HORLOGE_Secondes > 59)
	{
		V_HORLOGE_Secondes = 0;
		V_HORLOGE_Minutes += 1;
	}
	if (V_HORLOGE_Minutes > 59)
	{
		V_HORLOGE_Minutes = 0;
		V_HORLOGE_Heures += 1;
	}
	if (V_HORLOGE_Heures > 23)
	{
		V_HORLOGE_Heures = 0;
		V_HORLOGE_Jour += 1;
	}
	if (V_HORLOGE_Jour > 7) V_HORLOGE_Jour = 1;
	
	V_PROGRAMME_Jour = 0;
	lidiv=ldiv((24*(V_HORLOGE_Jour-1) + V_HORLOGE_Heures),8);
	X = (int)lidiv.quot;
	Y = (int)lidiv.rem;
	
	if ((bit_test(V_PROGRAMME_Hebdo[X],Y)) && (V_MODE_Regul == C_Auto)) V_PROGRAMME_Jour = 1;
}

/////////////////////////////////////////////////////////////
void Analyse_Etat_Touches()			// En sortie, V_TOUCHE_Selectionnee correspond  la touche qui a t appuye
												// et V_TOUCHE_AppuiLong = 1 si elle a t appuye longtemps
{
	V_TOUCHE_Selectionnee = C_Aucune;
	V_TOUCHE_AppuiLong = 0;
	
	if (TOUCHE_Jour) V_TOUCHE_Appuyee = C_TOUCHE_Jour;
		else
		{
			if (TOUCHE_Nuit) V_TOUCHE_Appuyee = C_TOUCHE_Nuit;
				else
				{
					if (TOUCHE_Mode) V_TOUCHE_Appuyee = C_TOUCHE_Mode;
						else
						{
							if (TOUCHE_Reglage) V_TOUCHE_Appuyee = C_TOUCHE_Reglage;
							else V_TOUCHE_Appuyee = C_Aucune;
						}
				}
		}
	if ((V_TOUCHE_Appuyee != C_Aucune) && (V_TOUCHE_Appuyee == V_TOUCHE_EtaitDejaAppuyee))
	{
		V_TOUCHE_AppuiDuree += 1;
		if (V_TOUCHE_AppuiDuree >= C_TOUCHE_DureeMiniAppuiLong)
		{
			V_TOUCHE_AppuiLong = 1;
			V_TOUCHE_Selectionnee = V_TOUCHE_Appuyee;
		}
	}
	else V_TOUCHE_AppuiDuree = 0;
	
	if (V_TOUCHE_Appuyee == C_Aucune) V_TOUCHE_Selectionnee = V_TOUCHE_EtaitDejaAppuyee;
	
	V_TOUCHE_EtaitDejaAppuyee = V_TOUCHE_Appuyee;
}	
/////////////////////////////////////////////////////////////
void Changement_Mode()				// Changement de mode de rgul, passage regul <=> reglage, chgt mode reglage
{
	int New_MODE;
	
	if (V_MODE == C_Regulation)				// Regulation -------------------
	{
		New_MODE = V_MODE_Regul;
		if (((V_MODE_Regul == C_NuitForce) || (V_MODE_Regul == C_JourForce)) && (V_TIMER_Force_Heure_Fin == V_HORLOGE_Heures))	New_MODE = C_Auto;
		if (ENTREE_OFFForce)
		{
			V_ModeOFFForce = 1;
		}
		else
		{
			V_ModeOFFForce = 0;
			if (((V_MODE_Regul == C_NuitForce) || (V_MODE_Regul == C_JourForce)) && (V_TOUCHE_Selectionnee == C_Touche_Mode))	New_MODE = C_Auto;
			if ((V_MODE_Regul == C_OFF) && (V_TOUCHE_Selectionnee == C_Touche_Mode))	New_MODE = C_Auto;
			if ((V_MODE_Regul == C_Auto) && (V_TOUCHE_Selectionnee == C_Touche_Mode))	New_MODE = C_ManuNuit;
			if (((V_MODE_Regul == C_ManuNuit) || (V_MODE_Regul == C_ManuJour)) && (V_TOUCHE_Selectionnee == C_Touche_Mode))	New_MODE = C_OFF;
			if ((V_MODE_Regul == C_ManuNuit) && (V_TOUCHE_Selectionnee == C_Touche_Jour))	New_MODE = C_ManuJour;
			if ((V_MODE_Regul == C_ManuJour) && (V_TOUCHE_Selectionnee == C_Touche_Nuit))	New_MODE = C_ManuNuit;
	
			if ((V_MODE_Regul == C_Auto) && (V_TOUCHE_Selectionnee == C_Touche_Nuit) && V_TOUCHE_AppuiLong)
			{
				New_MODE = C_NuitForce;
				V_TIMER_Force_Heure_Fin = V_HORLOGE_Heures + 1;
				if (V_TIMER_Force_Heure_Fin == 24) V_TIMER_Force_Heure_Fin = 0;
			}
			if ((V_MODE_Regul == C_Auto) && (V_TOUCHE_Selectionnee == C_Touche_Jour) && V_TOUCHE_AppuiLong)
			{
				New_MODE = C_JourForce;
				V_TIMER_Force_Heure_Fin = V_HORLOGE_Heures + 1;
				if (V_TIMER_Force_Heure_Fin == 24) V_TIMER_Force_Heure_Fin = 0;
			}
		}
		
		V_MODE_Regul = New_MODE;

		if (V_TOUCHE_Selectionnee == C_Touche_Reglage)
		{
			V_MODE = C_Reglage;
			V_PlageHoraire = 0;
			V_MODE_Reglage = V_HORLOGE_Jour - 1;		// Affiche directement le programme du jour pour permettre consultation rapide.
		}
	}
	else												// Reglage -------------------
	{		
		if (V_TOUCHE_Selectionnee == C_Touche_Mode)
		{
			V_MODE_Reglage++;
			if (V_MODE_Reglage > C_Hysteresis) V_MODE_Reglage = 0;
			V_PlageHoraire = 0;
		}
		if (V_TOUCHE_Selectionnee == C_Touche_Reglage)
		{
			// Sauvegarde des donnes en NVM
			NVM_Params_Save();
			
			// Retour en mode rgulation
			V_MODE = C_Regulation;
		}
	}
}

/////////////////////////////////////////////////////////////
void Actions_Touches()					// (autres que chgt mode)	
{
	ldiv_t lidiv;
	int X, Y;

	if (V_MODE == C_Regulation)
	{
		if ((V_MODE_Regul == C_NuitForce) && (V_TOUCHE_Selectionnee == C_Touche_Nuit) && (!V_TOUCHE_AppuiLong)) V_TIMER_Force_Heure_Fin += 1;
		if ((V_MODE_Regul == C_JourForce) && (V_TOUCHE_Selectionnee == C_Touche_Jour) && (!V_TOUCHE_AppuiLong)) V_TIMER_Force_Heure_Fin += 1;
		if (V_TIMER_Force_Heure_Fin == 24) V_TIMER_Force_Heure_Fin = 0;
	}
	else	// Pour chaque paramtre, C_Touche_Nuit => +1, C_Touche_Jour => -1 sauf pour Lundi -> Dimanche
	{
		switch (V_MODE_Reglage)
		{
			case C_TempNuit :
				if (V_TOUCHE_Selectionnee == C_Touche_Jour)
				{
					if (V_TEMPNUIT >= 29.9) V_TEMPNUIT = 29.9; else V_TEMPNUIT+=0.1;
				}
				if (V_TOUCHE_Selectionnee == C_Touche_Nuit)
				{
					if (V_TEMPNUIT <= 0.0) V_TEMPNUIT = 0.0; else V_TEMPNUIT-=0.1;
				}
				break;
			case C_TempJour :
				if (V_TOUCHE_Selectionnee == C_Touche_Jour)
				{
					if (V_TEMPJOUR >= 29.9) V_TEMPJOUR = 29.9; else V_TEMPJOUR+=0.1;
				}
				if (V_TOUCHE_Selectionnee == C_Touche_Nuit)
				{
					if (V_TEMPJOUR <= 0.0) V_TEMPJOUR = 0.0; else V_TEMPJOUR-=0.1;
				}
				break;
			case C_Jour :
				if (V_TOUCHE_Selectionnee == C_Touche_Jour)
				{
					if (V_HORLOGE_Jour == 7) V_HORLOGE_Jour = 1; else V_HORLOGE_Jour++;
				}
				if (V_TOUCHE_Selectionnee == C_Touche_Nuit)
				{
					if (V_HORLOGE_Jour == 1) V_HORLOGE_Jour = 7; else V_HORLOGE_Jour--;
				}
				break;
			case C_Heures :
				if (V_TOUCHE_Selectionnee == C_Touche_Jour)
				{
					if (V_HORLOGE_Heures == 23) V_HORLOGE_Heures = 0; else V_HORLOGE_Heures++;
				}
				if (V_TOUCHE_Selectionnee == C_Touche_Nuit)
				{
					if (V_HORLOGE_Heures == 0) V_HORLOGE_Heures = 23; else V_HORLOGE_Heures--;
				}
				break;
			case C_Minutes :
				if (V_TOUCHE_Selectionnee == C_Touche_Jour)
				{
					if (V_HORLOGE_Minutes == 59) V_HORLOGE_Minutes = 0; else V_HORLOGE_Minutes++;
				}
				if (V_TOUCHE_Selectionnee == C_Touche_Nuit)
				{
					if (V_HORLOGE_Minutes == 0) V_HORLOGE_Minutes = 59; else V_HORLOGE_Minutes--;
				}
				break;
			case C_Hysteresis :
				if (V_TOUCHE_Selectionnee == C_Touche_Jour)
				{
					if (V_REGUL_Hysteresis < 9) V_REGUL_Hysteresis++;
				}
				if (V_TOUCHE_Selectionnee == C_Touche_Nuit)
				{
					if (V_REGUL_Hysteresis > 2) V_REGUL_Hysteresis--;
				}
				break;
			default :		// C_Lundi  C_Dimanche
				lidiv=ldiv((24*V_MODE_Reglage + V_PlageHoraire),8);
				X = (int)lidiv.quot;
				Y = (int)lidiv.rem;
				if (V_TOUCHE_Selectionnee == C_Touche_Jour) bit_set(V_PROGRAMME_Hebdo[X],Y);
				if (V_TOUCHE_Selectionnee == C_Touche_Nuit) bit_clear(V_PROGRAMME_Hebdo[X],Y);
				if ((V_TOUCHE_Selectionnee == C_Touche_Jour) || (V_TOUCHE_Selectionnee == C_Touche_Nuit))
				{
					V_PlageHoraire++;
					if (V_PlageHoraire > 23) V_PlageHoraire = 0;
				}
		}
	}
}

/////////////////////////////////////////////////////////////
void Mesure_Temperature()
{
	long TempBrute;

		// 3V = 1024	= 50C
		// 0V = 0		= -25C

	TempBrute = Read_ADC();
	V_TEMPERATURE = ((float)V_TEMPERATURE*(C_TEMP_MoyGliss-1)+(float)TempBrute*(25+50)/1024-25.0)/C_TEMP_MoyGliss;
}

/////////////////////////////////////////////////////////////
void Choix_Temp_Consigne()
{
	switch (V_MODE_Regul)
	{
		case C_Auto :
			if (V_PROGRAMME_Jour) V_CONSIGNE_T = V_TEMPJOUR; else V_CONSIGNE_T = V_TEMPNUIT;
			break;
		case C_ManuNuit : V_CONSIGNE_T = V_TEMPNUIT; break;
		case C_ManuJour : V_CONSIGNE_T = V_TEMPJOUR; break;
		case C_NuitForce : V_CONSIGNE_T = V_TEMPNUIT; break;
		case C_JourForce : V_CONSIGNE_T = V_TEMPJOUR; break;
		default : break;
	}
}

/////////////////////////////////////////////////////////////
void MaJ_Regulation()
{
	if ((V_MODE_Regul != C_OFF) && (!V_ModeOFFForce))
	{	
//		if (V_TEMPERATURE > (V_CONSIGNE_T + (float)V_REGUL_Hysteresis/10)) Output_Low(RELAI_Pilote1);
//		if (V_TEMPERATURE < V_CONSIGNE_T) Output_High(RELAI_Pilote1);
		if (V_TEMPERATURE > (V_CONSIGNE_T + (float)V_REGUL_Hysteresis/20)) Output_Low(RELAI_Pilote1);
		if (V_TEMPERATURE < (V_CONSIGNE_T - (float)V_REGUL_Hysteresis/20)) Output_High(RELAI_Pilote1);
		Output_Low(RELAI_Pilote2);
	}
	else
	{
		Output_Low(RELAI_Pilote1);
		Output_High(RELAI_Pilote2);
	}
}

/////////////////////////////////////////////////////////////
void MAJ_Etat_LEDs()
{
	Output_Low(LED_Nuit);
	Output_Low(LED_Jour);
	
	switch (V_MODE_Regul)
	{
		case C_ManuNuit : Output_High(LED_Nuit); break;
		case C_ManuJour : Output_High(LED_Jour); break;
		case C_NuitForce : Output_Bit(LED_Nuit,V_HORLOGE_SecondePaire); break;
		case C_JourForce : Output_Bit(LED_Jour,V_HORLOGE_SecondePaire); break;
		case C_Auto :
			if (V_PROGRAMME_Jour) Output_High(LED_Jour); else Output_High(LED_Nuit);
			break;
		default : break;
	}
}

/////////////////////////////////////////////////////////////
void System_Init()
{
	// Direction des ports
	Set_Tris_A(0b00010001);		// Direction PORTA (1=entre)
	Set_Tris_B(0b00000000);		// Direction PORTB
	Set_Tris_C(0b00100111);		// Direction PORTC
	
	//Timer 1 (Squenceur de tches)
   setup_timer_1(T1_INTERNAL | T1_DIV_BY_2);
   set_timer1(65536-PERIODE_TIMER1);
   int_count = INTERRUPTS_PER_SECOND;
   enable_interrupts(INT_TIMER1);
	
	// Interruptions
   enable_interrupts(GLOBAL);

	// A/D	!!!! Modifier #DEVICE dans 16F876.h pour fixer nb bits acqui
	setup_adc_ports( AN0 );
		// NO_ANALOGS                           7    // None
		// ALL_ANALOG                           0    // A0 A1 A2 A3 A5         
		// AN0_AN1_AN2_AN4_VSS_VREF             1    // A0 A1 A2 A5 VRefh=A3              
		// AN0_AN1_AN3                          4    // A0 A1 A3
		// AN0_AN1_VSS_VREF                     5    // A0 A1 VRefh=A3
		// AN0_AN1_AN4_VREF_VREF             0x08    // A0 A1 A5 VRefh=A3 VRefl=A2              
		// AN0_AN1_VREF_VREF                 0x0D    // A0 A1 VRefh=A3 VRefl=A2
		// AN0                               0x0E    // A0
		// AN0_VREF_VREF                     0x0F    // A0 VRefh=A3 VRefl=A2
		// ANALOG_RA3_REF         0x1         //!old only provided for compatibility
		// RA0_RA1_RA3_ANALOG     0x4         //!old only provided for compatibility
		// RA0_RA1_ANALOG_RA3_REF 0x5         //!old only provided for compatibility
		// ANALOG_RA3_RA2_REF              0x8   //!old only provided for compatibility
		// RA0_RA1_ANALOG_RA3_RA2_REF      0xD   //!old only provided for compatibility
		// RA0_ANALOG                      0xE   //!old only provided for compatibility
		// RA0_ANALOG_RA3_RA2_REF          0xF   //!old only provided for compatibility

	setup_adc( ADC_CLOCK_INTERNAL );
		// ADC_OFF                0          // ADC Off
		// ADC_CLOCK_DIV_2    0x100
		// ADC_CLOCK_DIV_8     0x40
		// ADC_CLOCK_DIV_32    0x80
		// ADC_CLOCK_INTERNAL  0xc0          // Internal 2-6us

	set_adc_channel( 0 );		// Prochaine(s) acqui(s) sur AN0
	
	// Reset E/S
	Output_Low(LED_Jour);
	Output_Low(LED_Nuit);
	Output_Low(RELAI_Pilote1);
	Output_Low(RELAI_Pilote2);

	// Ecran LCD
	NOKIA_Init();
	
	// Lecture des paramtres en NVM
	NVM_Params_Read();
	
	// Init variables
	V_HORLOGE_Heures = 0;
	V_HORLOGE_Minutes = 0;
	V_HORLOGE_Secondes = 0;
	V_HORLOGE_Jour = 1;
	V_HORLOGE_SecondePaire = 0;
				
	V_TEMPERATURE = 0.0;
	V_CONSIGNE_T = 0;
				
	V_MODE = C_Regulation;
	V_ModeOFFForce = 0;
	V_MODE_Regul = C_ManuNuit;
	V_MODE_Reglage = C_Lundi;
	V_PROGRAMME_Jour = 0;
	V_TIMER_Force_Heure_Fin = 0;
	V_TOUCHE_Appuyee = C_Aucune;
	V_TOUCHE_EtaitDejaAppuyee = C_Aucune;
	V_TOUCHE_Selectionnee = C_Aucune;
	V_TOUCHE_AppuiDuree = 0;
	V_TOUCHE_AppuiLong = 0;
}	


/////////////////////////////////////////////////////////////
void Affiche_Jour_et_Heure()
{
	switch (V_HORLOGE_Jour)
	{
		case 1 : NOKIA_WriteChar(" Lun "); break;
		case 2 : NOKIA_WriteChar(" Mar "); break;
		case 3 : NOKIA_WriteChar(" Mer "); break;
		case 4 : NOKIA_WriteChar(" Jeu "); break;
		case 5 : NOKIA_WriteChar(" Ven "); break;
		case 6 : NOKIA_WriteChar(" Sam "); break;
		case 7 : NOKIA_WriteChar(" Dim "); break;
	} 
	sprintf(Str,"%02d:%02d:%02d ",V_HORLOGE_Heures, V_HORLOGE_Minutes, V_HORLOGE_Secondes);
	NOKIA_WriteString(Str);
}

/////////////////////////////////////////////////////////////
#INT_TIMER1
void Gestion_Sequenceur()
{
   set_timer1(65536-PERIODE_TIMER1);
	// 1 Hz
	if(--int_count==0)
	{
		Incremente_Horloge();				// Incrm Sec, Min, Heures, Jour
		Mesure_Temperature();
		int_count = INTERRUPTS_PER_SECOND;
    }
	
	// 10 Hz
	Analyse_Etat_Touches();
	Changement_Mode();
	Actions_Touches();	// (autres que chgt mode)	
	Choix_Temp_Consigne();
	MaJ_Regulation();
	MAJ_Etat_LEDs();
}

/////////////////////////////////////////////////////////////
///////////////////        Main     /////////////////////////
/////////////////////////////////////////////////////////////
void main()
{
	int Heure;
	ldiv_t lidiv;
	int X, Y;
	
	System_Init();

	// Boucle principale
	while(1)						// Gestion de l'affichage
	{
		NOKIA_GotoXY(0,4);
		for (X=0 ; X<14 ; X++) NOKIA_WriteChar(0);
			
		if (V_MODE == C_Regulation)		// Mode REGULATION ------------
		{
			NOKIA_GotoXY(0,0);
			Affiche_Jour_et_Heure();
				
			sprintf(Str," %4.1f C",V_Temperature);
			//NOKIA_GotoXY(0,2);
			//NOKIA_WriteString(Str);
			NOKIA_WriteBigString(Str,0,2);
		
			NOKIA_GotoXY(0,5);
			if (V_ModeOFFForce)
			{
				NOKIA_WriteChar("   Off (Ext)  ");
			}
			else
			{
				switch (V_MODE_Regul)
				{
					case C_Off :			NOKIA_WriteChar("     Off      "); break;
					case C_Auto :			NOKIA_WriteChar(" Auto ");	break;
					case C_ManuNuit :		NOKIA_WriteChar(" Manu "); break;
					case C_ManuJour :		NOKIA_WriteChar(" Manu "); break;
					case C_NuitForce :	NOKIA_WriteChar("(Auto)"); break;
					case C_JourForce :	NOKIA_WriteChar("(Auto)"); break;
				}
				if ((V_MODE_Regul == C_Auto) || (V_MODE_Regul == C_ManuNuit) || (V_MODE_Regul == C_ManuJour))		// Ecriture de la Temp de consigne
				{
					NOKIA_GotoXY(6,5);
					NOKIA_WriteChar(0x01);
					sprintf(Str," %4.1f C",V_CONSIGNE_T);
					NOKIA_GotoXY(7,5);
					NOKIA_WriteString(Str);
				}
				
				if ((V_MODE_Regul == C_NuitForce) || (V_MODE_Regul == C_JourForce))		// Ecriture de l'heure de fin
				{
					NOKIA_GotoXY(6,5);
					NOKIA_WriteChar(0x01);
					sprintf(Str,"  %02d:00 ",V_TIMER_Force_Heure_Fin);
					NOKIA_GotoXY(7,5);
					NOKIA_WriteString(Str);
				}
			}
						
			NOKIA_GotoXY(0,1);
			NOKIA_WriteChar("              ");
		}
		else										// Mode REGLAGE ------------
		{
			if (V_MODE_Reglage <= C_Dimanche)		// Programme ------
			{
				NOKIA_GotoXY(0,0);
				NOKIA_WriteChar("Programmation ");
				
				switch (V_MODE_Reglage)
				{
					case C_Lundi :		NOKIA_WriteChar("     Lundi    ");	break;
					case C_Mardi :		NOKIA_WriteChar("     Mardi    ");	break;
					case C_Mercredi :	NOKIA_WriteChar("   Mercredi   ");	break;
					case C_Jeudi :		NOKIA_WriteChar("     Jeudi    ");	break;
					case C_Vendredi :	NOKIA_WriteChar("   Vendredi   ");	break;
					case C_Samedi :	NOKIA_WriteChar("    Samedi    ");	break;
					case C_Dimanche :	NOKIA_WriteChar("   Dimanche   ");	break;
				}
				
				NOKIA_GotoXY(0,2);
				NOKIA_WriteChar("AM");
				for (Heure=0; Heure<24; Heure++)
				{
					if (heure ==12) NOKIA_WriteChar("PM");
					if (V_PlageHoraire == Heure) NOKIA_WriteChar(0x03); // Plage en cours de modif
					else
					{
						lidiv=ldiv((24*V_MODE_Reglage + Heure),8);
						X = (int)lidiv.quot;
						Y = (int)lidiv.rem;
						if (bit_test(V_PROGRAMME_Hebdo[X],Y)) NOKIA_WriteChar(0x12); else NOKIA_WriteChar(0x02);
					}
				}
			}
			else													// Autres ------
			{
				NOKIA_GotoXY(0,0);
				if (V_MODE_Reglage == C_TempNuit)
				{
					NOKIA_WriteChar("     Nuit     ");
					
					sprintf(Str,"    %4.1f C    ",V_TEMPNUIT);
					NOKIA_GotoXY(0,2);
					NOKIA_WriteString(Str);
				}
				if (V_MODE_Reglage == C_TempJour)
				{
					NOKIA_WriteChar("     Jour     ");
					
					sprintf(Str,"    %4.1f C    ",V_TEMPJOUR);
					NOKIA_GotoXY(0,2);
					NOKIA_WriteString(Str);
				}
				if ((V_MODE_Reglage == C_Jour) || (V_MODE_Reglage == C_Heures) || (V_MODE_Reglage == C_Minutes))
				{
					NOKIA_WriteChar("     Date     ");
					NOKIA_GotoXY(0,2);
					Affiche_Jour_et_Heure();
				}
		
				if (V_MODE_Reglage == C_Hysteresis)
				{
					NOKIA_WriteChar("Hysteresis Reg");
					
					sprintf(Str,"    0.%1d C     ",V_REGUL_Hysteresis);
					NOKIA_GotoXY(0,2);
					NOKIA_WriteString(Str);
				} 
			NOKIA_GotoXY(0,1);
			NOKIA_WriteChar("              ");
			NOKIA_GotoXY(0,3);
			NOKIA_WriteChar("              ");
			NOKIA_GotoXY(0,5);
			NOKIA_WriteChar("    Reglage   ");
			}
		}
	}
}
